# -*- coding: utf-8
"""
select {([Measures].[Customer Count]), ([Measures].[Sales Count])} 
               * {([Time].[1997].[Q2].[4]), ([Time].[1997].[Q2].[5]) } on columns, 
        [Gender].[Gender].ALLMEMBERS on rows 
        from [Sales]

"""

result={'Axes': {'Axis': [{'Tuples': {'Tuple': [{'Member': [{'Caption': 'Customer '
                                                                 'Count',
                                                      'DisplayInfo': '0',
                                                      'LName': '[Measures].[MeasuresLevel]',
                                                      'LNum': '0',
                                                      'UName': '[Measures].[Customer '
                                                               'Count]',
                                                      '_Hierarchy': 'Measures',
                                                      'text': None},
                                                     {'Caption': '4',
                                                      'DisplayInfo': '0',
                                                      'LName': '[Time].[Month]',
                                                      'LNum': '2',
                                                      'UName': '[Time].[1997].[Q2].[4]',
                                                      '_Hierarchy': 'Time',
                                                      'text': None}],
                                          'text': None},
                                         {'Member': [{'Caption': 'Customer '
                                                                 'Count',
                                                      'DisplayInfo': '0',
                                                      'LName': '[Measures].[MeasuresLevel]',
                                                      'LNum': '0',
                                                      'UName': '[Measures].[Customer '
                                                               'Count]',
                                                      '_Hierarchy': 'Measures',
                                                      'text': None},
                                                     {'Caption': '5',
                                                      'DisplayInfo': '131072',
                                                      'LName': '[Time].[Month]',
                                                      'LNum': '2',
                                                      'UName': '[Time].[1997].[Q2].[5]',
                                                      '_Hierarchy': 'Time',
                                                      'text': None}],
                                          'text': None},
                                         {'Member': [{'Caption': 'Sales Count',
                                                      'DisplayInfo': '0',
                                                      'LName': '[Measures].[MeasuresLevel]',
                                                      'LNum': '0',
                                                      'UName': '[Measures].[Sales '
                                                               'Count]',
                                                      '_Hierarchy': 'Measures',
                                                      'text': None},
                                                     {'Caption': '4',
                                                      'DisplayInfo': '131072',
                                                      'LName': '[Time].[Month]',
                                                      'LNum': '2',
                                                      'UName': '[Time].[1997].[Q2].[4]',
                                                      '_Hierarchy': 'Time',
                                                      'text': None}],
                                          'text': None},
                                         {'Member': [{'Caption': 'Sales Count',
                                                      'DisplayInfo': '0',
                                                      'LName': '[Measures].[MeasuresLevel]',
                                                      'LNum': '0',
                                                      'UName': '[Measures].[Sales '
                                                               'Count]',
                                                      '_Hierarchy': 'Measures',
                                                      'text': None},
                                                     {'Caption': '5',
                                                      'DisplayInfo': '131072',
                                                      'LName': '[Time].[Month]',
                                                      'LNum': '2',
                                                      'UName': '[Time].[1997].[Q2].[5]',
                                                      '_Hierarchy': 'Time',
                                                      'text': None}],
                                          'text': None}],
                               'text': None},
                    '_name': 'Axis0',
                    'text': None},
                   {'Tuples': {'Tuple': [{'Member': {'Caption': 'F',
                                                     'DisplayInfo': '0',
                                                     'LName': '[Gender].[Gender]',
                                                     'LNum': '1',
                                                     'UName': '[Gender].[F]',
                                                     '_Hierarchy': 'Gender',
                                                     'text': None},
                                          'text': None},
                                         {'Member': {'Caption': 'M',
                                                     'DisplayInfo': '131072',
                                                     'LName': '[Gender].[Gender]',
                                                     'LNum': '1',
                                                     'UName': '[Gender].[M]',
                                                     '_Hierarchy': 'Gender',
                                                     'text': None},
                                          'text': None}],
                               'text': None},
                    '_name': 'Axis1',
                    'text': None},
                   {'Tuples': {'Tuple': None, 'text': None},
                    '_name': 'SlicerAxis',
                    'text': None}],
          'text': None},
 'CellData': {'Cell': [{'FmtValue': '657',
                        'FormatString': '#,###',
                        'Value': 657,
                        '_CellOrdinal': '0',
                        'text': None},
                       {'FmtValue': '672',
                        'FormatString': '#,###',
                        'Value': 672,
                        '_CellOrdinal': '1',
                        'text': None},
                       {'FmtValue': '3.251',
                        'FormatString': '#,###',
                        'Value': 3251,
                        '_CellOrdinal': '2',
                        'text': None},
                       {'FmtValue': '3.438',
                        'FormatString': '#,###',
                        'Value': 3438,
                        '_CellOrdinal': '3',
                        'text': None},
                       {'FmtValue': '660',
                        'FormatString': '#,###',
                        'Value': 660,
                        '_CellOrdinal': '4',
                        'text': None},
                       {'FmtValue': '699',
                        'FormatString': '#,###',
                        'Value': 699,
                        '_CellOrdinal': '5',
                        'text': None},
                       {'FmtValue': '3.339',
                        'FormatString': '#,###',
                        'Value': 3339,
                        '_CellOrdinal': '6',
                        'text': None},
                       {'FmtValue': '3.428',
                        'FormatString': '#,###',
                        'Value': 3428,
                        '_CellOrdinal': '7',
                        'text': None}],
              'text': None},
 'OlapInfo': {'AxesInfo': {'AxisInfo': [{'HierarchyInfo': [{'Caption': {'_name': '[Measures].[MEMBER_CAPTION]',
                                                                        'text': None},
                                                            'DisplayInfo': {'_name': '[Measures].[DISPLAY_INFO]',
                                                                            'text': None},
                                                            'LName': {'_name': '[Measures].[LEVEL_UNIQUE_NAME]',
                                                                      'text': None},
                                                            'LNum': {'_name': '[Measures].[LEVEL_NUMBER]',
                                                                     'text': None},
                                                            'UName': {'_name': '[Measures].[MEMBER_UNIQUE_NAME]',
                                                                      'text': None},
                                                            '_name': 'Measures',
                                                            'text': None},
                                                           {'Caption': {'_name': '[Time].[MEMBER_CAPTION]',
                                                                        'text': None},
                                                            'DisplayInfo': {'_name': '[Time].[DISPLAY_INFO]',
                                                                            'text': None},
                                                            'LName': {'_name': '[Time].[LEVEL_UNIQUE_NAME]',
                                                                      'text': None},
                                                            'LNum': {'_name': '[Time].[LEVEL_NUMBER]',
                                                                     'text': None},
                                                            'UName': {'_name': '[Time].[MEMBER_UNIQUE_NAME]',
                                                                      'text': None},
                                                            '_name': 'Time',
                                                            'text': None}],
                                         '_name': 'Axis0',
                                         'text': None},
                                        {'HierarchyInfo': {'Caption': {'_name': '[Gender].[MEMBER_CAPTION]',
                                                                       'text': None},
                                                           'DisplayInfo': {'_name': '[Gender].[DISPLAY_INFO]',
                                                                           'text': None},
                                                           'LName': {'_name': '[Gender].[LEVEL_UNIQUE_NAME]',
                                                                     'text': None},
                                                           'LNum': {'_name': '[Gender].[LEVEL_NUMBER]',
                                                                    'text': None},
                                                           'UName': {'_name': '[Gender].[MEMBER_UNIQUE_NAME]',
                                                                     'text': None},
                                                           '_name': 'Gender',
                                                           'text': None},
                                         '_name': 'Axis1',
                                         'text': None},
                                        {'_name': 'SlicerAxis', 'text': None}],
                           'text': None},
              'CellInfo': {'FmtValue': {'_name': 'FORMATTED_VALUE',
                                        'text': None},
                           'FormatString': {'_name': 'FORMAT_STRING',
                                            'text': None},
                           'Value': {'_name': 'VALUE', 'text': None},
                           'text': None},
              'CubeInfo': {'Cube': {'CubeName': 'Sales', 'text': None},
                           'text': None},
              'text': None},
 'text': None}

xml_response="""<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Header>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
<cxmla:ExecuteResponse xmlns:cxmla="urn:schemas-microsoft-com:xml-analysis">
  <cxmla:return>
    <root xmlns="urn:schemas-microsoft-com:xml-analysis:mddataset" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:EX="urn:schemas-microsoft-com:xml-analysis:exception">
      <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:schemas-microsoft-com:xml-analysis:mddataset" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:sql="urn:schemas-microsoft-com:xml-sql" targetNamespace="urn:schemas-microsoft-com:xml-analysis:mddataset" elementFormDefault="qualified">
        <xsd:complexType name="MemberType">
          <xsd:sequence>
            <xsd:element name="UName" type="xsd:string"/>
            <xsd:element name="Caption" type="xsd:string"/>
            <xsd:element name="LName" type="xsd:string"/>
            <xsd:element name="LNum" type="xsd:unsignedInt"/>
            <xsd:element name="DisplayInfo" type="xsd:unsignedInt"/>
            <xsd:sequence maxOccurs="unbounded" minOccurs="0">
              <xsd:any processContents="lax" maxOccurs="unbounded"/>
            </xsd:sequence>
          </xsd:sequence>
          <xsd:attribute name="Hierarchy" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="PropType">
          <xsd:attribute name="name" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="TupleType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Member" type="MemberType"/>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="MembersType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Member" type="MemberType"/>
          </xsd:sequence>
          <xsd:attribute name="Hierarchy" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="TuplesType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Tuple" type="TupleType"/>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="CrossProductType">
          <xsd:sequence>
            <xsd:choice minOccurs="0" maxOccurs="unbounded">
              <xsd:element name="Members" type="MembersType"/>
              <xsd:element name="Tuples" type="TuplesType"/>
            </xsd:choice>
          </xsd:sequence>
          <xsd:attribute name="Size" type="xsd:unsignedInt"/>
        </xsd:complexType>
        <xsd:complexType name="OlapInfo">
          <xsd:sequence>
            <xsd:element name="CubeInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:element name="Cube" maxOccurs="unbounded">
                    <xsd:complexType>
                      <xsd:sequence>
                        <xsd:element name="CubeName" type="xsd:string"/>
                      </xsd:sequence>
                    </xsd:complexType>
                  </xsd:element>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
            <xsd:element name="AxesInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:element name="AxisInfo" maxOccurs="unbounded">
                    <xsd:complexType>
                      <xsd:sequence>
                        <xsd:element name="HierarchyInfo" minOccurs="0" maxOccurs="unbounded">
                          <xsd:complexType>
                            <xsd:sequence>
                              <xsd:sequence maxOccurs="unbounded">
                                <xsd:element name="UName" type="PropType"/>
                                <xsd:element name="Caption" type="PropType"/>
                                <xsd:element name="LName" type="PropType"/>
                                <xsd:element name="LNum" type="PropType"/>
                                <xsd:element name="DisplayInfo" type="PropType" minOccurs="0" maxOccurs="unbounded"/>
                              </xsd:sequence>
                              <xsd:sequence>
                                <xsd:any processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                              </xsd:sequence>
                            </xsd:sequence>
                            <xsd:attribute name="name" type="xsd:string" use="required"/>
                          </xsd:complexType>
                        </xsd:element>
                      </xsd:sequence>
                      <xsd:attribute name="name" type="xsd:string"/>
                    </xsd:complexType>
                  </xsd:element>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
            <xsd:element name="CellInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:sequence minOccurs="0" maxOccurs="unbounded">
                    <xsd:choice>
                      <xsd:element name="Value" type="PropType"/>
                      <xsd:element name="FmtValue" type="PropType"/>
                      <xsd:element name="BackColor" type="PropType"/>
                      <xsd:element name="ForeColor" type="PropType"/>
                      <xsd:element name="FontName" type="PropType"/>
                      <xsd:element name="FontSize" type="PropType"/>
                      <xsd:element name="FontFlags" type="PropType"/>
                      <xsd:element name="FormatString" type="PropType"/>
                      <xsd:element name="NonEmptyBehavior" type="PropType"/>
                      <xsd:element name="SolveOrder" type="PropType"/>
                      <xsd:element name="Updateable" type="PropType"/>
                      <xsd:element name="Visible" type="PropType"/>
                      <xsd:element name="Expression" type="PropType"/>
                    </xsd:choice>
                  </xsd:sequence>
                  <xsd:sequence maxOccurs="unbounded" minOccurs="0">
                    <xsd:any processContents="lax" maxOccurs="unbounded"/>
                  </xsd:sequence>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="Axes">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Axis">
              <xsd:complexType>
                <xsd:choice minOccurs="0" maxOccurs="unbounded">
                  <xsd:element name="CrossProduct" type="CrossProductType"/>
                  <xsd:element name="Tuples" type="TuplesType"/>
                  <xsd:element name="Members" type="MembersType"/>
                </xsd:choice>
                <xsd:attribute name="name" type="xsd:string"/>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="CellData">
          <xsd:sequence>
            <xsd:element name="Cell" minOccurs="0" maxOccurs="unbounded">
              <xsd:complexType>
                <xsd:sequence maxOccurs="unbounded">
                  <xsd:choice>
                    <xsd:element name="Value"/>
                    <xsd:element name="FmtValue" type="xsd:string"/>
                    <xsd:element name="BackColor" type="xsd:unsignedInt"/>
                    <xsd:element name="ForeColor" type="xsd:unsignedInt"/>
                    <xsd:element name="FontName" type="xsd:string"/>
                    <xsd:element name="FontSize" type="xsd:unsignedShort"/>
                    <xsd:element name="FontFlags" type="xsd:unsignedInt"/>
                    <xsd:element name="FormatString" type="xsd:string"/>
                    <xsd:element name="NonEmptyBehavior" type="xsd:unsignedShort"/>
                    <xsd:element name="SolveOrder" type="xsd:unsignedInt"/>
                    <xsd:element name="Updateable" type="xsd:unsignedInt"/>
                    <xsd:element name="Visible" type="xsd:unsignedInt"/>
                    <xsd:element name="Expression" type="xsd:string"/>
                  </xsd:choice>
                </xsd:sequence>
                <xsd:attribute name="CellOrdinal" type="xsd:unsignedInt" use="required"/>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:element name="root">
          <xsd:complexType>
            <xsd:sequence maxOccurs="unbounded">
              <xsd:element name="OlapInfo" type="OlapInfo"/>
              <xsd:element name="Axes" type="Axes"/>
              <xsd:element name="CellData" type="CellData"/>
            </xsd:sequence>
          </xsd:complexType>
        </xsd:element>
      </xsd:schema>
      <OlapInfo>
        <CubeInfo>
          <Cube>
            <CubeName>Sales</CubeName>
          </Cube>
        </CubeInfo>
        <AxesInfo>
          <AxisInfo name="Axis0">
            <HierarchyInfo name="Measures">
              <UName name="[Measures].[MEMBER_UNIQUE_NAME]"/>
              <Caption name="[Measures].[MEMBER_CAPTION]"/>
              <LName name="[Measures].[LEVEL_UNIQUE_NAME]"/>
              <LNum name="[Measures].[LEVEL_NUMBER]"/>
              <DisplayInfo name="[Measures].[DISPLAY_INFO]"/>
            </HierarchyInfo>
            <HierarchyInfo name="Time">
              <UName name="[Time].[MEMBER_UNIQUE_NAME]"/>
              <Caption name="[Time].[MEMBER_CAPTION]"/>
              <LName name="[Time].[LEVEL_UNIQUE_NAME]"/>
              <LNum name="[Time].[LEVEL_NUMBER]"/>
              <DisplayInfo name="[Time].[DISPLAY_INFO]"/>
            </HierarchyInfo>
          </AxisInfo>
          <AxisInfo name="Axis1">
            <HierarchyInfo name="Gender">
              <UName name="[Gender].[MEMBER_UNIQUE_NAME]"/>
              <Caption name="[Gender].[MEMBER_CAPTION]"/>
              <LName name="[Gender].[LEVEL_UNIQUE_NAME]"/>
              <LNum name="[Gender].[LEVEL_NUMBER]"/>
              <DisplayInfo name="[Gender].[DISPLAY_INFO]"/>
            </HierarchyInfo>
          </AxisInfo>
          <AxisInfo name="SlicerAxis"/>
        </AxesInfo>
        <CellInfo>
          <Value name="VALUE"/>
          <FmtValue name="FORMATTED_VALUE"/>
          <FormatString name="FORMAT_STRING"/>
        </CellInfo>
      </OlapInfo>
      <Axes>
        <Axis name="Axis0">
          <Tuples>
            <Tuple>
              <Member Hierarchy="Measures">
                <UName>[Measures].[Customer Count]</UName>
                <Caption>Customer Count</Caption>
                <LName>[Measures].[MeasuresLevel]</LName>
                <LNum>0</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
              <Member Hierarchy="Time">
                <UName>[Time].[1997].[Q2].[4]</UName>
                <Caption>4</Caption>
                <LName>[Time].[Month]</LName>
                <LNum>2</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
            </Tuple>
            <Tuple>
              <Member Hierarchy="Measures">
                <UName>[Measures].[Customer Count]</UName>
                <Caption>Customer Count</Caption>
                <LName>[Measures].[MeasuresLevel]</LName>
                <LNum>0</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
              <Member Hierarchy="Time">
                <UName>[Time].[1997].[Q2].[5]</UName>
                <Caption>5</Caption>
                <LName>[Time].[Month]</LName>
                <LNum>2</LNum>
                <DisplayInfo>131072</DisplayInfo>
              </Member>
            </Tuple>
            <Tuple>
              <Member Hierarchy="Measures">
                <UName>[Measures].[Sales Count]</UName>
                <Caption>Sales Count</Caption>
                <LName>[Measures].[MeasuresLevel]</LName>
                <LNum>0</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
              <Member Hierarchy="Time">
                <UName>[Time].[1997].[Q2].[4]</UName>
                <Caption>4</Caption>
                <LName>[Time].[Month]</LName>
                <LNum>2</LNum>
                <DisplayInfo>131072</DisplayInfo>
              </Member>
            </Tuple>
            <Tuple>
              <Member Hierarchy="Measures">
                <UName>[Measures].[Sales Count]</UName>
                <Caption>Sales Count</Caption>
                <LName>[Measures].[MeasuresLevel]</LName>
                <LNum>0</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
              <Member Hierarchy="Time">
                <UName>[Time].[1997].[Q2].[5]</UName>
                <Caption>5</Caption>
                <LName>[Time].[Month]</LName>
                <LNum>2</LNum>
                <DisplayInfo>131072</DisplayInfo>
              </Member>
            </Tuple>
          </Tuples>
        </Axis>
        <Axis name="Axis1">
          <Tuples>
            <Tuple>
              <Member Hierarchy="Gender">
                <UName>[Gender].[F]</UName>
                <Caption>F</Caption>
                <LName>[Gender].[Gender]</LName>
                <LNum>1</LNum>
                <DisplayInfo>0</DisplayInfo>
              </Member>
            </Tuple>
            <Tuple>
              <Member Hierarchy="Gender">
                <UName>[Gender].[M]</UName>
                <Caption>M</Caption>
                <LName>[Gender].[Gender]</LName>
                <LNum>1</LNum>
                <DisplayInfo>131072</DisplayInfo>
              </Member>
            </Tuple>
          </Tuples>
        </Axis>
        <Axis name="SlicerAxis">
          <Tuples>
            <Tuple/>
          </Tuples>
        </Axis>
      </Axes>
      <CellData>
        <Cell CellOrdinal="0">
          <Value xsi:type="xsd:int">657</Value>
          <FmtValue>657</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="1">
          <Value xsi:type="xsd:int">672</Value>
          <FmtValue>672</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="2">
          <Value xsi:type="xsd:int">3251</Value>
          <FmtValue>3.251</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="3">
          <Value xsi:type="xsd:int">3438</Value>
          <FmtValue>3.438</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="4">
          <Value xsi:type="xsd:int">660</Value>
          <FmtValue>660</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="5">
          <Value xsi:type="xsd:int">699</Value>
          <FmtValue>699</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="6">
          <Value xsi:type="xsd:int">3339</Value>
          <FmtValue>3.339</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
        <Cell CellOrdinal="7">
          <Value xsi:type="xsd:int">3428</Value>
          <FmtValue>3.428</FmtValue>
          <FormatString>#,###</FormatString>
        </Cell>
      </CellData>
    </root>
  </cxmla:return>
</cxmla:ExecuteResponse>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
"""


'''
Created on 18.04.2012
- For SSAS tests are done against the Adventure Works DW 2008R2.
- For Mondrian tests are done against the FoodMart sample data source.

@author: norman

n.b.:
  Mondrian is running with a mysql backend. 
  Using the distributed embedded derby backend fails in MDSchemaMembers with an internal error (when
  a subselect is executed in the read-only container where the database lives).
  
'''

import unittest
import olap.xmla.xmla as xmla
from nose.tools import *
from requests.auth import HTTPBasicAuth
import tests.integration.execute_mondrian as execute_mondrian
import tests.integration.execute_ssas as execute_ssas
import tests.mockhelper as mockhelper

mock_location = "mock://somewhere/over/the/rainbow"

mondrian={
    "type":"mondrian",
    "location":"http://localhost:8080/xmondrian/xmla",
    "auth": None,
    "catalog":"FoodMart",
    "cube":"[Sales]",
    "set1":"[Measures].ALLMEMBERS",
    "set2":"[Time].[1997].[Q2].children",
    "set3":"[Gender].[Gender].ALLMEMBERS",
    "conversation":execute_mondrian,
}
ssas={
	"type":"ssas",
	"location":"http://dwh-bi/olap/msmdpump.dll",
    "auth" : None,
	"catalog":"Adventure Works DW 2008R2",
	"cube":"[Sales Summary]",
	"set1":"[Measures].ALLMEMBERS",
	"set2":"[Date].[Month of Year].ALLMEMBERS",
	"set3":"[Product].[Product Categories].[Category].ALLMEMBERS",
    "conversation":execute_ssas,
}

class XMLAExecute(object):
    be = None
    logreq = None
    record = False

    def setUp(self):
        testname = self.id().split(".")[-1]
        session = mockhelper.mockedsession(self.be["conversation"], testname)

        self.cube = self.be["cube"]
        self.set1 = self.be["set1"]
        self.set2 = self.be["set2"]
        self.set3 = self.be["set3"] 
        self.catalog = self.be["catalog"]
        self.p = xmla.XMLAProvider()
        if self.logreq:
            self.logreq.prefix=testname
        kw = {
            "log":self.logreq,
            "session":session
        }
        self.c = self.p.connect(location=self.be["location"], auth=self.be["auth"], **kw)
        
    def tearDown(self):
        if self.logreq and self.record:
            self.logreq.saveConversation(fname="try_execute_"+self.be["type"] +".py")

    def test2Axes(self):
        cmd= """select {%(set1)s} * {%(set2)s} on columns, 
                       %(set3)s on rows 
                from %(cube)s""" %\
	    {"set1":self.set1, "set2":self.set2, "set3":self.set3, 
	     "cube":self.cube}

        erg = self.c.Execute(cmd, Catalog=self.catalog)
        erg.getSlice(property="Value")

    def test3Axes(self):
        cmd= """select %(set1)s on columns, 
                       %(set2)s on rows, %(set3)s on Axis(2) 
                from %(cube)s""" %\
	    {"set1":self.set1, "set2":self.set2, "set3":self.set3, 
	     "cube":self.cube}
        erg = self.c.Execute(cmd, Catalog=self.catalog)
        erg.getSlice(property="Value")

    def testNoAxesButOneCell(self): 
    # well there is a sliceraxis of course ...
        cmd= "select from %(cube)s" % {"cube":self.cube}
        erg = self.c.Execute(cmd, Catalog=self.catalog)
        erg.getSlice(property="Value")

    def testOneDimensional(self): 
	# columns with cells but no rows
        cmd= "select %(set1)s on columns from %(cube)s" %\
	    {"cube":self.cube, "set1":self.set1}
        erg = self.c.Execute(cmd, Catalog=self.catalog)
        erg.getSlice(property="Value")

    def testOneDimensional2(self): 
        # empty column with no cells but rows
        if self.be["type"] == "mondrian": return # NPE in Mondrian's backend
        cmd= "select {} on columns, %(set1)s on rows from %(cube)s" %\
	    {"cube":self.cube, "set1":self.set1}
        erg = self.c.Execute(cmd, Catalog=self.catalog)
        erg.getSlice(property="Value")
		

try:
    from testconfig import config
    server=config['xmla']['server'] or ""
    server = server.split(",")
    for server_section in server:
        if server_section in globals():
            globals()[server_section].update(config.get(server_section, {}))

    do_record=(config['xmla']['record'] or "no") == "yes"

    if "ssas" in server:
        from requests_kerberos import HTTPKerberosAuth
        ssas["auth"] = HTTPKerberosAuth()

except:
    # we mock responses
    server=["mondrian", "ssas"]
    config = {}
    mondrian["location"]=mock_location
    ssas["location"]=mock_location
    do_record = False

if "mondrian" in server:
    class TestMondrian(XMLAExecute, unittest.TestCase):
        be = mondrian
        logreq = mockhelper.LogRequest(False)
        record = do_record

if "ssas" in server:
    class TestSSAS(XMLAExecute, unittest.TestCase):
        be = ssas
        logreq = mockhelper.LogRequest(False)
        record = do_record
        




import unittest
import sys
import olap.xmla.utils as utils

class TestUtils(unittest.TestCase):

    def testaslist(self):
        for x in ["Blah", None, {}, 7, (), set([1,2,3]), True]:
            self.assertIsInstance( utils.aslist(x), list)

        x = ["blah"]
        self.assertEqual( utils.aslist(x), x)

    def testschemaNameToMethodName(self):
        mn=utils.schemaNameToMethodName
        self.assertEqual(mn("DBSCHEMA_palim"), "getDBSchemaPalim")
        self.assertEqual(mn("MDSCHEMA_palim"), "getMDSchemaPalim")
        self.assertEqual(mn("DISCOVER_palim"), "getPalim")
        self.assertEqual(mn("I_KNOW_WHAT_YOU_DID_LAST_SPRINGBREAK"), 
                         "getIKnowWhatYouDidLastSpringbreak")

    def testFromETree(self):
        import lxml.etree as etree
        root = etree.Element("root")
        c1 = etree.SubElement(root, "c1")
        c1.text="iam_c1"
        c11 = etree.SubElement(c1, "c11")
        c11.text="iam_c11"
        c12 = etree.SubElement(c1, "c12", attrib={"a1":"val_a1", "a2":"val_a2"})
        c12.text="iam_c12"
        c2 = etree.SubElement(root, "c2")
        c2.text="iam_c2"
        d = utils.fromETree(root, ns="")
        self.assertEqual(d.c1.text, "iam_c1")
        self.assertEqual(d.c1.c12.text, "iam_c12")
        self.assertEqual(d.c1.c12._a1, "val_a1")

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
import olap.xmla.xmla as xmla
import olap.xmla.utils as utils
import pprint
import os.path
from requests_kerberos import HTTPKerberosAuth
from requests.auth import HTTPBasicAuth
from zeep import Plugin

class LogRequest(Plugin):
    def __init__(self):
        self.res=""

    def ingress(self, envelope, http_headers, operation):
        self.res=utils.etree_tostring(envelope)

def main(mdxfile, location, catalog, krb, username, password, spn, sslverify):
    p = os.path.dirname(os.path.realpath(__file__))
    pyfile = os.path.join(p, os.path.splitext(mdxfile)[0] + os.path.extsep + "py")
    cmd = open(os.path.join(p,mdxfile)).read()

    p = xmla.XMLAProvider()

    auth = None
    if krb:
        kw = {}
        if spn:
            service, host = spn.split("@",1)
            kw["service"]=service
            kw["hostname_override"]=host
        if username:
            kw["principal"] = username
        auth = HTTPKerberosAuth(**kw)
    elif username:
        auth = HTTPBasicAuth(username, password)
    
    kwargs = {}
    log = LogRequest()
    kwargs["log"] = log
    c=p.connect(location=location, sslverify=sslverify, auth=auth, **kwargs)
    res=c.Execute(cmd, Catalog=catalog)
    x=utils.dictify(res.root, keep_none_text=True)

    erg=pprint.pformat(x)
    encodinghint="# -*- coding: utf-8"
    open(pyfile, "w+").write('%s\n"""\n%s\n"""\n\nresult=%s\n\nxml_response="""%s"""' % (encodinghint, cmd, erg, log.res))

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="MDX Resultwriter.")
    parser.add_argument("mdxfile")
    parser.add_argument("--location", dest="location", 
                        help="location von XMLASource", 
                        default="http://localhost:8080/xmondrian/xmla")
    parser.add_argument("--catalog", dest="catalog", 
                        help="catalog to execute the mdx in", 
                        default="FoodMart")
    parser.add_argument("--username", dest="username", help="connect as user", 
                        default=None)
    parser.add_argument("--password", dest="password", help="use password", 
                        default=None)
    parser.add_argument("--krb", action="store_const", const=True, default=False, dest="krb", 
                        help="kerberos auth")
    parser.add_argument("--spn", dest="spn", 
                        help="spn, defaults to HTTP@host if omitted")
    parser.add_argument("--sslverify", dest="sslverify", default=True,
                        help="sslverify, either False or path of cert file")
    args = parser.parse_args()

    if isinstance(args.sslverify, str):
        if args.sslverify.upper() in ["0", "FALSE", "NO", "NEIN"]:
            args.sslverify = False

    main(args.mdxfile, args.location, args.catalog, args.krb,
         args.username, args.password, args.spn, args.sslverify)




'''
Created on 18.04.2012

@author: norman
'''
from .interfaces import XMLAException
from zeep import Client, Plugin, xsd
from zeep.exceptions import Fault
from zeep.transports import Transport

#import types
from .formatreader import TupleFormatReader, DAXFormatReader
from .utils import *
import logging

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


schema_xmla="urn:schemas-microsoft-com:xml-analysis"
schema_xmla_rowset="urn:schemas-microsoft-com:xml-analysis:rowset"
schema_xmla_mddataset="urn:schemas-microsoft-com:xml-analysis:mddataset"
schema_soap_env="http://schemas.xmlsoap.org/soap/envelope/"

# the following along with changes to the wsdl (elementFormDefault="unqualified") is needed
# to make it fly with icCube, which expects elements w/o namespace prefix
class LogRequest(Plugin):
    def __init__(self, enabled=True):
        self.enabled = enabled

    def egress(self, envelope, http_headers, operation, binding_options):
        if self.enabled:
            print(etree_tostring(envelope))

    def ingress(self, envelope, http_headers, operation):
        if self.enabled:
            print(etree_tostring(envelope))

    def enable(self):
        self.enabled=True
    def disable(self):
        self.enabled=False

class SessionPlugin(Plugin):
  def __init__(self, xmlaconn):
      self.xmlaconn = xmlaconn

  def ingress(self, envelope, http_headers, operation):
    #print(etree_tostring(envelope))
    if self.xmlaconn.getListenOnSessionId():
        nsmap={'se': schema_soap_env,
               'xmla': schema_xmla}
        s=envelope.xpath("/se:Envelope/se:Header/xmla:Session", namespaces=nsmap)[0]
        sid=s.attrib.get("SessionId")
        self.xmlaconn.setSessionId(sid)


# lsit of XMLA1.1 rowsets: 
xmla1_1_rowsets = ["DISCOVER_DATASOURCES",
                   "DISCOVER_PROPERTIES", 
                   "DISCOVER_SCHEMA_ROWSETS",
                   "DISCOVER_ENUMERATORS",
                   "DISCOVER_LITERALS",
                   "DISCOVER_KEYWORDS",
                   "DBSCHEMA_CATALOGS",
                   "DBSCHEMA_COLUMNS",
                   "DBSCHEMA_TABLES",
                   "DBSCHEMA_TABLES_INFO",
                   "DBSCHEMA_PROVIDER_TYPES",
                   "MDSCHEMA_ACTIONS",
                   "MDSCHEMA_CUBES",
                   "MDSCHEMA_DIMENSIONS",
                   "MDSCHEMA_FUNCTIONS",
                   "MDSCHEMA_HIERARCHIES",
                   "MDSCHEMA_MEASURES",
                   "MDSCHEMA_MEMBERS",
                   "MDSCHEMA_PROPERTIES",
                   "MDSCHEMA_SETS"
                   ]

class XMLAConnection(object):
    
    @classmethod
    def addMethod(cls, funcname, func):
        return setattr(cls, funcname, func)

        
    @classmethod
    def setupMembers(cls):
        def getFunc(schemaName):
            return lambda this, *args, **kw: cls.Discover(this, 
                                                          schemaName, 
                                                          *args, **kw)
        
        for schemaName in xmla1_1_rowsets:
            mname = schemaNameToMethodName(schemaName)
            cls.addMethod( mname, getFunc(schemaName) )

    def __init__(self, url, location, sslverify, **kwargs):

        if "session" in kwargs:
            session = kwargs["session"]
            del kwargs["session"]
            transport = Transport(session=session)
        else:
            transport = Transport()
            
        if "auth" in kwargs:
            transport.session.auth = kwargs["auth"]
            del kwargs["auth"]

        transport.session.verify = sslverify
        self.sessionplugin=SessionPlugin(self)
        plugins=[self.sessionplugin]

        if "log" in kwargs:
            log = kwargs.get("log")
            if isinstance(log, Plugin):
                plugins.append(log)
            elif log == True:
                plugins.append(LogRequest())
            del kwargs["log"]
            
        self.client = Client(url, 
                             transport=transport, 
                             # cache=None, unwrap=False,
                             plugins=plugins)

        self.service = self.client.create_service(ns_name(schema_xmla,"MsXmlAnalysisSoap"), location)
        self.client.set_ns_prefix(None, schema_xmla)
        # optional, call might fail
        self.getMDSchemaLevels = lambda *args, **kw: self.Discover("MDSCHEMA_LEVELS", 
                                                                   *args, **kw)
        self.setListenOnSessionId(False)
        self.setSessionId(None)
        self._soapheaders=None
             

    def getListenOnSessionId(self):
        return self.listenOnSessionId

    def setListenOnSessionId(self, trueOrFalse):
        self.listenOnSessionId = trueOrFalse

    def setSessionId(self, sessionId):
        self.sessionId = sessionId
        
    def Discover(self, what, restrictions=None, properties=None):
        rl = as_etree(restrictions, "RestrictionList")
        pl = as_etree(properties, "PropertyList")
        try:
            #import pdb; pdb.set_trace()
            doc=self.service.Discover(RequestType=what, Restrictions=rl, Properties=pl, _soapheaders=self._soapheaders)
            root = fromETree(doc.body["return"]["_value_1"], ns=schema_xmla_rowset)
            res = getattr(root, "row", [])
            if res:
                res = aslist(res)
        except Fault as fault:
            raise XMLAException(fault.message, dictify(fromETree(fault.detail, ns=None)))
        #logger.debug( res )
        return res


    def Execute(self, command, dimformat="Multidimensional", 
                axisFormat="TupleFormat", **kwargs):
        if isinstance(command, stringtypes):
            command=as_etree({"Statement": command})
        props = {"Format":dimformat, "AxisFormat": axisFormat}
        props.update(kwargs)

        plist = as_etree({"PropertyList": props})
        try:
            res = self.service.Execute(Command=command, Properties=plist, _soapheaders=self._soapheaders)
            root_raw = res.body["return"]["_value_1"]
            if root_raw.tag.startswith("{{{}}}".format(schema_xmla_rowset)):
                return DAXFormatReader(root_raw, fromETree(root_raw, ns=schema_xmla_rowset))
            else:
                return TupleFormatReader(root_raw)
        except Fault as fault:
            raise XMLAException(fault.message, dictify(fromETree(fault.detail, ns=None)))
        
        
    def BeginSession(self):
        bs= self.client.get_element(ns_name(schema_xmla,"BeginSession"))(mustUnderstand=1)
        self.setListenOnSessionId(True)
        cmd = as_etree("Statement")

        self.service.Execute(Command=cmd,_soapheaders={"BeginSession":bs})
        self.setListenOnSessionId(False)

        sess= self.client.get_element(ns_name(schema_xmla,"Session"))(SessionId=self.sessionId, mustUnderstand = 1)
        self._soapheaders={"Session":sess}
        
    def EndSession(self):
        if self.sessionId is not None:
            es= self.client.get_element(ns_name(schema_xmla,"EndSession"))(SessionId=self.sessionId, mustUnderstand = 1)
            cmd = as_etree("Statement")
            self.service.Execute(Command=cmd, _soapheaders={"EndSession":es})
            self.setSessionId(None)
            self._soapheaders=None

                
XMLAConnection.setupMembers()



'''
Created on 18.04.2012

SSAS Tests are done against the Adventure Works DW 2008R2, 
tested against ssas version 10.50.2500.0 (comes with sqlserver 2008 R2)
Mondrian was test with version 3.3.0.14703

@author: norman

n.b.:
  Mondrian is running with a mysql backend. 
  Using the embedded derby backend (as distributed by mondrian) fails in 
  MDSchemaMembers with an internal error (when a subselect is executed in 
  the read-only container where the database lives).
  
'''

import unittest
from nose.tools import *
import olap.xmla.xmla as xmla
from requests.auth import HTTPBasicAuth

from olap.xmla.connection import xmla1_1_rowsets
import tests.mockhelper as mockhelper
import tests.integration.discover_mondrian as discover_mondrian
import tests.integration.discover_ssas as discover_ssas

mock_location = "mock://somewhere/over/the/rainbow"

mondrian={
    "type":"mondrian",
    "auth": None,
    "location":"http://localhost:8080/xmondrian/xmla",
    # Datasourcename depends on distribution
    #"ds": "Provider=Mondrian;DataSource=MondrianFoodMart;",
    "ds": "Foodmart",
    "catalog":"FoodMart",
    "restrict_cube":"HR",
    "restrict_dim":"Position",
    "restrict_unique_dim":"[Gender]",
    "cubes_expected":7,
    "restrict_funcname":"||",
    "restrict_hier":"Time",
    "restrict_level_unique_name":"[Employees].[Employee Id]",
    "restrict_hierarchy_unique_name":"[Time]",
    "cube_measures":5,
    "schema_levels":3,
    "schema_sets":1,
    "schema_sets_needs_cubename":False,
    "schema_tables":1,
    "flow_catalog":"FoodMart",
    "flow_cube":"Sales",
    "flow_hier":"[Product]",
    "flow_level":"[Product].[Product Category]",
    "flow_member":"[Product].[Drink].[Alcoholic Beverages].[Beer and Wine]",
    "flow_member_name":"Beer and Wine",
    "conversation":discover_mondrian,
}

ssas={
    "type":"ssas",
    "location":"http://dwh-bi/olap/msmdpump.dll",
    "auth" : None,
    "ds":"DWH-BI",
    "catalog":"Adventure Works DW 2008R2",
    "restrict_cube":"Adventure Works",
    "restrict_dim":"Account",
    "restrict_unique_dim":"[Account]",
    "cubes_expected":7,
    "restrict_funcname":"TRIM",
    "restrict_hier":"Account Number",
    "restrict_level_unique_name":"[Customer].[Customer Geography].[Country]",
    "restrict_hierarchy_unique_name":"[Customer].[Customer Geography]",
    "cube_measures": 51,
    "schema_levels":6,
    "schema_sets":1,
    "schema_sets_needs_cubename":True, # not really, but if you have a few DBs on your server this will bring it to it's knees
    "schema_tables":1,
    "flow_catalog":"Adventure Works DW 2008R2",
    "flow_cube":"Adventure Works",
    "flow_hier":"[Product].[Category]",
    "flow_level":"[Product].[Category].[Category]",
    "flow_member":"[Product].[Category].&[3]",
    "flow_member_name":"Clothing",
    "conversation":discover_ssas
}

import logging
#logging.basicConfig(level=logging.INFO)
#logging.getLogger('suds.client').setLevel(logging.DEBUG)


class XMLA(object):
    be = None
    logreq = None
    record = False

    def setUp(self):
        testname = self.id().split(".")[-1]
        session = mockhelper.mockedsession(self.be["conversation"], testname)
        self.p = xmla.XMLAProvider()
        if self.logreq:
            self.logreq.prefix=testname
        kw = {
            "log":self.logreq,
            "session":session
        }
        self.c = self.p.connect(location=self.be["location"], 
                                auth=self.be["auth"], **kw)
        #self.c.BeginSession()
        self.getSchemaRowsetSupport()
         
    def getSchemaRowsetSupport(self):
        if self.supported is None:
            self.supported=[x["SchemaName"] for x in self.c.getSchemaRowsets()]
            self.proprietary = [x for x in self.supported if not(x in xmla1_1_rowsets)]
            self.conform = [x for x in self.supported if (x in xmla1_1_rowsets)]
            self.unsupported = [x for x in xmla1_1_rowsets if not (x in self.supported)]
    
    def tearDown(self):
        #self.c.EndSession()
        if self.logreq and self.record:
            self.logreq.saveConversation(fname="try_discover_"+self.be["type"] +".py")
    
    def testGetDatasources(self):
        #self.log.enable()
        erg=self.c.getDatasources()
        #e = erg[0]
        #print(type(e))
        #print(e)
        self.assertTrue(len(erg) == 1, "One Datasource is expected")
        self.assertEqual(self.be["ds"], erg[0]["DataSourceName"])
        
    def testGetProperties(self):
        erg=self.c.getProperties()
        # check for required xmla properties
        req = """AxisFormat,BeginRange,Catalog,Content,Cube,DataSourceInfo,
                 EndRange,Format,LocaleIdentifier,MDXSupport,Password,
                 ProviderName,ProviderVersion,StateSupport,Timeout,UserName"""
        # of those required icCube only returns ProviderVersion and Catalog
        # if self.be == iccube: req = "Catalog,ProviderVersion"
        propnames = [x["PropertyName"] for x in erg]
        for p in req.split(","):
            self.assertIn(p.strip(), propnames)

    def testGetDBSchemaCatalogs(self):
        #self.log.enable()
        erg=self.c.getDBSchemaCatalogs()
        self.assertTrue(len(erg) > 0, "One Catalog is expected - at least")
        self.assertIn(self.be["catalog"], [x["CATALOG_NAME"] for x in erg])
        
    
    def testGetDBSchemaColumns(self):
        if "DBSCHEMA_COLUMNS" in self.conform:
            erg=self.c.getDBSchemaColumns()
            self.assertTrue(len(erg) > 0, 
                            "There must be at least one column, really!")
        
    def testGetDBSchemaTables(self):
        erg=self.c.getDBSchemaTables()
        self.assertTrue(len(erg) >= self.be["schema_tables"], 
                        "There must be at least one table")

    def testGetDBSchemaProviderTypes(self):
        if "DBSCHEMA_PROVIDER_TYPES" in self.conform:
            erg=self.c.getDBSchemaProviderTypes()
            msg="There should be at least one type like INTEGER for example"
            self.assertTrue(len(erg) > 0, msg)
        
    def testGetDBSchemaTablesInfo(self):
        if "DBSCHEMA_TABLES_INFO" in self.conform:
            erg=self.c.getDBSchemaTablesInfo()
            self.assertTrue(len(erg) > 0, 
                            "There must be at least one tablesinfo")

    def testGetMDSchemaActions(self):
        erg=self.c.getMDSchemaActions(
            restrictions={"CUBE_NAME":self.be["restrict_cube"], "COORDINATE":self.be["restrict_cube"], "COORDINATE_TYPE":1})
        self.assertTrue(len(erg)==0, "no actions expected")

    def testGetMDSchemaCubes(self):
        # oh my, mondrian doesn't know what do do with Catalog in the Proplist ... 
        # but only when requesting cubes, other rowset requests are going 
        # through just fine (are probably ignored on the server side)
        props = {"Catalog":self.be["catalog"]}
        erg=self.c.getMDSchemaCubes(properties=props)
        self.assertEqual(len(erg), self.be["cubes_expected"])
        #self.log.enable()
        erg=self.c.getMDSchemaCubes(
            restrictions={"CUBE_NAME":self.be["restrict_cube"]}, 
            properties=props)
        self.assertEqual(len(erg), 1)

    def testGetSchemaRowsets(self):
        erg=self.c.getSchemaRowsets()
        self.assertTrue(len(erg)>0, "at least one schema is expected")
        
    def testGetMDSchemaFunctions(self):
        erg=self.c.getMDSchemaFunctions()
        self.assertTrue(len(erg)> 1, 
                        "There should be more than one function description.")
        erg=self.c.getMDSchemaFunctions(
            restrictions={"FUNCTION_NAME":self.be["restrict_funcname"]})
        self.assertEqual(len(erg), 1)
        
    def testGetMDSchemaMembers(self):
        erg=self.c.getMDSchemaMembers(
            restrictions={"CUBE_NAME":self.be["restrict_cube"], 
                          "LEVEL_UNIQUE_NAME":self.be["restrict_level_unique_name"]}, 
            properties={"Catalog":self.be["catalog"]})
        self.assertTrue(len(erg)> 1, 
                        "There should be more than one dimension member.")

    def testGetMDSchemaProperties(self):
        erg=self.c.getMDSchemaProperties(
            {"CUBE_NAME":self.be["restrict_cube"], 
             'LEVEL_UNIQUE_NAME': self.be["restrict_level_unique_name"]}, 
            properties={"Catalog":self.be["catalog"]})
        self.assertTrue(len(erg)> 1, 
                        "There should be more than one schema property.")

    def testGetMDSchemaSets(self):
        if self.be["schema_sets_needs_cubename"]:
            erg=self.c.getMDSchemaSets(restrictions={"CUBE_NAME":self.be["restrict_cube"]})
        else:
            erg=self.c.getMDSchemaSets()
        
        self.assertTrue(len(erg)>= self.be["schema_sets"], 
                        "There should be a schema set.")

    def testGetEnumerators(self):
        if "DISCOVER_ENUMERATORS" in self.conform:
            erg = self.c.getEnumerators()
            msg = "There should be at least one enumerator."
            self.assertTrue(len(erg)> 0, msg)
        
    def testGetLiterals(self):
        erg = self.c.getLiterals()
        self.assertTrue(len(erg)> 0, "There should be at least one literal.")
        
    def testGetKeywords(self):
        # instead of not listing the KEYWORDS Rowset in a call to 
        # DISCOVER_SCHEMA_ROWSETS, iccube throws an exceptions
        if "DISCOVER_KEYWORDS" in self.conform:
            erg = self.c.getKeywords()
            self.assertTrue(len(erg)> 0, 
                            "There should be at least one keyword.")
        

    def testGetMDSchemaDimensions(self):
        erg=self.c.getMDSchemaDimensions(
            restrictions={"CUBE_NAME":self.be["restrict_cube"], 
                          "DIMENSION_NAME":self.be["restrict_dim"]},
            properties={"Catalog":self.be["catalog"]})
        self.assertEqual(len(erg), 1)
        
    def testGetMDSchemaHierarchies(self):
        erg=self.c.getMDSchemaHierarchies(
            restrictions={"CUBE_NAME":self.be["restrict_cube"]}, 
            properties={"Catalog":self.be["catalog"]})
        self.assertTrue(len(erg)> 1, 
                        "There should be more than one hierarchy.")
        erg=self.c.getMDSchemaHierarchies(
            restrictions={"HIERARCHY_NAME":self.be["restrict_hier"], 
                          "CUBE_NAME":self.be["restrict_cube"]}, 
            properties={"Catalog":self.be["catalog"]})
        self.assertEqual(len(erg), 1)

    def testGetMDSchemaMeasures(self):
        erg=self.c.getMDSchemaMeasures(
            restrictions={"CUBE_NAME":self.be["restrict_cube"]}, 
            properties={"Catalog":self.be["catalog"]})
        self.assertEqual(len(erg), self.be["cube_measures"])

    def testGetSchemaLevels(self):
        #self.log.enable()
        erg = self.c.getMDSchemaLevels(
            restrictions={"CUBE_NAME":self.be["restrict_cube"], 
             'HIERARCHY_UNIQUE_NAME': self.be["restrict_hierarchy_unique_name"]},
            properties={"Catalog":self.be["catalog"]})
        self.assertEqual(len(erg), self.be["schema_levels"])
    
    def testGetDBSchemaSchemata(self):
        if "DBSCHEMA_SCHEMATA" in self.supported:
            erg = self.c.Discover("DBSCHEMA_SCHEMATA")

    def testFlow(self):
        cat=self.c.getCatalog(self.be["flow_catalog"])
        cube=cat.getCube(self.be["flow_cube"])
        hier=cube.getHierarchy(self.be["flow_hier"])
        level=hier.getLevel(self.be["flow_level"])
        member=level.getMember(self.be["flow_member"])
        self.assertEqual(member.MEMBER_NAME, self.be["flow_member_name"])

try:
    from testconfig import config
    server=config['xmla']['server'] or ""
    server = server.split(",")
    for server_section in server:
        if server_section in globals():
            globals()[server_section].update(config.get(server_section, {}))

    do_record=(config['xmla']['record'] or "no") == "yes"
    if "ssas" in server:
        from requests_kerberos import HTTPKerberosAuth
        ssas["auth"] = HTTPKerberosAuth()
            
except:
    # we mock responses
    server=["mondrian", "ssas"]
    config = {}
    mondrian["location"]=mock_location
    ssas["location"]=mock_location
    do_record = False

if "mondrian" in server:
    class TestMondrian(XMLA, unittest.TestCase):
        be = mondrian
        supported = proprietary = conform = unsupported = None
        logreq = mockhelper.LogRequest(False)
        record = do_record

if "ssas" in server:
    class TestSSAS(XMLA, unittest.TestCase):
        be = ssas
        supported = proprietary = conform = unsupported = None
        logreq = mockhelper.LogRequest(False)
        record = do_record




# -*- coding: utf-8
"""
-- newer versions of mondrian now return a sliceraxis, i removed axisinformation in the result by hand here
select from [Sales]

"""

result={'Axes': '',
 'CellData': {'Cell': {'FmtValue': '266.773',
                       'FormatString': 'Standard',
                       'Value': 266773.0,
                       '_CellOrdinal': '0'}},
 'OlapInfo': {'AxesInfo': {'AxisInfo': {'_name': 'SlicerAxis'}},
              'CellInfo': {'FmtValue': {'_name': 'FORMATTED_VALUE'},
                           'FormatString': {'_name': 'FORMAT_STRING'},
                           'Value': {'_name': 'VALUE'}},
              'CubeInfo': {'Cube': {'CubeName': 'Sales'}}}}

xml_response="""<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Header>
</SOAP-ENV:Header>
<SOAP-ENV:Body>
<cxmla:ExecuteResponse xmlns:cxmla="urn:schemas-microsoft-com:xml-analysis">
  <cxmla:return>
    <root xmlns="urn:schemas-microsoft-com:xml-analysis:mddataset" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:EX="urn:schemas-microsoft-com:xml-analysis:exception">
      <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:schemas-microsoft-com:xml-analysis:mddataset" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:sql="urn:schemas-microsoft-com:xml-sql" targetNamespace="urn:schemas-microsoft-com:xml-analysis:mddataset" elementFormDefault="qualified">
        <xsd:complexType name="MemberType">
          <xsd:sequence>
            <xsd:element name="UName" type="xsd:string"/>
            <xsd:element name="Caption" type="xsd:string"/>
            <xsd:element name="LName" type="xsd:string"/>
            <xsd:element name="LNum" type="xsd:unsignedInt"/>
            <xsd:element name="DisplayInfo" type="xsd:unsignedInt"/>
            <xsd:sequence maxOccurs="unbounded" minOccurs="0">
              <xsd:any processContents="lax" maxOccurs="unbounded"/>
            </xsd:sequence>
          </xsd:sequence>
          <xsd:attribute name="Hierarchy" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="PropType">
          <xsd:attribute name="name" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="TupleType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Member" type="MemberType"/>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="MembersType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Member" type="MemberType"/>
          </xsd:sequence>
          <xsd:attribute name="Hierarchy" type="xsd:string"/>
        </xsd:complexType>
        <xsd:complexType name="TuplesType">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Tuple" type="TupleType"/>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="CrossProductType">
          <xsd:sequence>
            <xsd:choice minOccurs="0" maxOccurs="unbounded">
              <xsd:element name="Members" type="MembersType"/>
              <xsd:element name="Tuples" type="TuplesType"/>
            </xsd:choice>
          </xsd:sequence>
          <xsd:attribute name="Size" type="xsd:unsignedInt"/>
        </xsd:complexType>
        <xsd:complexType name="OlapInfo">
          <xsd:sequence>
            <xsd:element name="CubeInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:element name="Cube" maxOccurs="unbounded">
                    <xsd:complexType>
                      <xsd:sequence>
                        <xsd:element name="CubeName" type="xsd:string"/>
                      </xsd:sequence>
                    </xsd:complexType>
                  </xsd:element>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
            <xsd:element name="AxesInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:element name="AxisInfo" maxOccurs="unbounded">
                    <xsd:complexType>
                      <xsd:sequence>
                        <xsd:element name="HierarchyInfo" minOccurs="0" maxOccurs="unbounded">
                          <xsd:complexType>
                            <xsd:sequence>
                              <xsd:sequence maxOccurs="unbounded">
                                <xsd:element name="UName" type="PropType"/>
                                <xsd:element name="Caption" type="PropType"/>
                                <xsd:element name="LName" type="PropType"/>
                                <xsd:element name="LNum" type="PropType"/>
                                <xsd:element name="DisplayInfo" type="PropType" minOccurs="0" maxOccurs="unbounded"/>
                              </xsd:sequence>
                              <xsd:sequence>
                                <xsd:any processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                              </xsd:sequence>
                            </xsd:sequence>
                            <xsd:attribute name="name" type="xsd:string" use="required"/>
                          </xsd:complexType>
                        </xsd:element>
                      </xsd:sequence>
                      <xsd:attribute name="name" type="xsd:string"/>
                    </xsd:complexType>
                  </xsd:element>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
            <xsd:element name="CellInfo">
              <xsd:complexType>
                <xsd:sequence>
                  <xsd:sequence minOccurs="0" maxOccurs="unbounded">
                    <xsd:choice>
                      <xsd:element name="Value" type="PropType"/>
                      <xsd:element name="FmtValue" type="PropType"/>
                      <xsd:element name="BackColor" type="PropType"/>
                      <xsd:element name="ForeColor" type="PropType"/>
                      <xsd:element name="FontName" type="PropType"/>
                      <xsd:element name="FontSize" type="PropType"/>
                      <xsd:element name="FontFlags" type="PropType"/>
                      <xsd:element name="FormatString" type="PropType"/>
                      <xsd:element name="NonEmptyBehavior" type="PropType"/>
                      <xsd:element name="SolveOrder" type="PropType"/>
                      <xsd:element name="Updateable" type="PropType"/>
                      <xsd:element name="Visible" type="PropType"/>
                      <xsd:element name="Expression" type="PropType"/>
                    </xsd:choice>
                  </xsd:sequence>
                  <xsd:sequence maxOccurs="unbounded" minOccurs="0">
                    <xsd:any processContents="lax" maxOccurs="unbounded"/>
                  </xsd:sequence>
                </xsd:sequence>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="Axes">
          <xsd:sequence maxOccurs="unbounded">
            <xsd:element name="Axis">
              <xsd:complexType>
                <xsd:choice minOccurs="0" maxOccurs="unbounded">
                  <xsd:element name="CrossProduct" type="CrossProductType"/>
                  <xsd:element name="Tuples" type="TuplesType"/>
                  <xsd:element name="Members" type="MembersType"/>
                </xsd:choice>
                <xsd:attribute name="name" type="xsd:string"/>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:complexType name="CellData">
          <xsd:sequence>
            <xsd:element name="Cell" minOccurs="0" maxOccurs="unbounded">
              <xsd:complexType>
                <xsd:sequence maxOccurs="unbounded">
                  <xsd:choice>
                    <xsd:element name="Value"/>
                    <xsd:element name="FmtValue" type="xsd:string"/>
                    <xsd:element name="BackColor" type="xsd:unsignedInt"/>
                    <xsd:element name="ForeColor" type="xsd:unsignedInt"/>
                    <xsd:element name="FontName" type="xsd:string"/>
                    <xsd:element name="FontSize" type="xsd:unsignedShort"/>
                    <xsd:element name="FontFlags" type="xsd:unsignedInt"/>
                    <xsd:element name="FormatString" type="xsd:string"/>
                    <xsd:element name="NonEmptyBehavior" type="xsd:unsignedShort"/>
                    <xsd:element name="SolveOrder" type="xsd:unsignedInt"/>
                    <xsd:element name="Updateable" type="xsd:unsignedInt"/>
                    <xsd:element name="Visible" type="xsd:unsignedInt"/>
                    <xsd:element name="Expression" type="xsd:string"/>
                  </xsd:choice>
                </xsd:sequence>
                <xsd:attribute name="CellOrdinal" type="xsd:unsignedInt" use="required"/>
              </xsd:complexType>
            </xsd:element>
          </xsd:sequence>
        </xsd:complexType>
        <xsd:element name="root">
          <xsd:complexType>
            <xsd:sequence maxOccurs="unbounded">
              <xsd:element name="OlapInfo" type="OlapInfo"/>
              <xsd:element name="Axes" type="Axes"/>
              <xsd:element name="CellData" type="CellData"/>
            </xsd:sequence>
          </xsd:complexType>
        </xsd:element>
      </xsd:schema>
      <OlapInfo>
        <CubeInfo>
          <Cube>
            <CubeName>Sales</CubeName>
          </Cube>
        </CubeInfo>
        <AxesInfo>
          <AxisInfo name="SlicerAxis"/>
        </AxesInfo>
        <CellInfo>
          <Value name="VALUE"/>
          <FmtValue name="FORMATTED_VALUE"/>
          <FormatString name="FORMAT_STRING"/>
        </CellInfo>
      </OlapInfo>
      <Axes></Axes>
      <CellData>
        <Cell CellOrdinal="0">
          <Value xsi:type="xsd:double">266773</Value>
          <FmtValue>266.773</FmtValue>
          <FormatString>Standard</FormatString>
        </Cell>
      </CellData>
    </root>
  </cxmla:return>
</cxmla:ExecuteResponse>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
"""


import six
from lxml import etree
from lxml.etree import QName

stringtypes = six.string_types
_UnmarshallableType=(type(None), six.string_types, six.integer_types, float, bool)
u=six.u

schema_instance="http://www.w3.org/2001/XMLSchema-instance"
class Data(dict):

    def __getattr__(self, name):
        if name in self:
            return self[name]
        return getattr(super(Data, self), name)

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        del self[name]


def aslist(something):
    """If something is not a list already make it one, otherwise simply return something"""
    return something if isinstance(something, list) else [something]

def dictify(r, keep_none_text=False):

    if isinstance(r, list):
        return [dictify(x, keep_none_text) for x in r]
    if isinstance(r, Data):
        d = {}
        for (k,v) in r.items():
            if k == "text" and v is None and not keep_none_text: continue
            d[k] = dictify(v, keep_none_text)
        return d
    if isinstance(r, dict):
        d = {}
        for (k,v) in r.items():
            d[k] = dictify(v, keep_none_text)
        return d
    return r


def schemaNameToMethodName(schemaName):
    """
    Convert a schema name like DISCOVER_SCHEMA_ROWSETS into a method name like getSchemaRowsets.
    1. split into parts by _
    2. replace: 
        DBSCHEMA => DBSchema 
        MDSCHEMA => MDSchema
        DISCOVER =>
        otherwise => lower case + capitalize
    3. join results and prepend with "get"
    """
    parts = schemaName.split("_")
    def replace(what):
       
        if what == "DBSCHEMA": return "DBSchema"
        elif what == "MDSCHEMA": return "MDSchema"
        elif what == "DISCOVER": return ""
            
        return what.lower().capitalize()

    return "get" + "".join(map(replace, parts))
            

class PropDict(dict):
    def __getattribute__(self, name):
        try:
            value = self[name]
        except:
            return dict.__getattribute__(self, name)
        if isinstance(value, dict):
            value = PropDict(value)
            self[name] = value
            return value
        elif isinstance(value, list):
            value = [PropDict(x) for x in value]
            self[name] = value
            return value
        return value

    def __delattr__(self, name):
        try:
            value = self[name]
            del self[name]
        except:
            super(PropDict,self).__delattr__(name)
        #try:
        #    value = self[name]
        #    super(PropDict,self).__delattr__(name)
        #except:
        #    super(PropDict,self).__delattr__(name)

def etree_tostring(et):
    return etree.tostring(et, pretty_print=True).decode("utf-8")

def as_etree(dict_or_str, parent_if_content=None):

    top = {}
    if isinstance(dict_or_str, stringtypes):
        top = {dict_or_str:None}
    elif isinstance(dict_or_str, dict):
        top = dict_or_str
        if isinstance(parent_if_content, stringtypes) and top:
            top = {parent_if_content:top}

    elems = []
    for (elem_name, elem_value) in top.items():
        
        elem=etree.Element(elem_name)
        if isinstance(elem_value, dict):
            subelem = as_etree(elem_value)
            if subelem is not None:
                if isinstance(subelem, list):
                    for c in subelem:
                        elem.append(c)
                else:
                    elem.append(subelem)
        else:
            if elem_value is not None:
                elem.text = str(elem_value)
        elems.append(elem)

    res = None
    if len(elems)==1:
        res = elems[0]
    elif len(elems)>1:
        res = elems
    return res

def ns_name(ns, name):
    if ns is None:
        return name
    return "{{{}}}{}".format(ns,name)

def fromETree(e, ns):
    p = Data()
    nst = ns_name(ns, "*")
    valtype = ns_name(schema_instance, "type")
    if e is not None:
        for (k,v) in e.attrib.items():
            setattr(p, "_"+k, v)
        p.text = e.text
        if p.text and p.text.strip() == "":
            p.text=None
    else:
        p.text=None
    if e is not None:
        if valtype in e.attrib:
            if e.attrib[valtype] in ["xsd:int", "xsd:unsignedInt"]:
                p.text = int(p.text)  
                delattr(p, "_"+valtype)
            elif e.attrib[valtype] in ["xsd:long"]:
                p.text = int(p.text)  if six.PY3 else long(p.text)
                delattr(p, "_"+valtype)
            elif e.attrib[valtype] in ["xsd:double", "xsd:float"]:
                p.text = float(p.text)  
                delattr(p, "_"+valtype)
        for c in e.findall(nst):
            t = QName(c)
            
            cd = fromETree(c, ns)
            #if len(cd.__dict__) == 1:
            if len(cd) == 1:
                cd = cd.text
            v = getattr(p, t.localname, None)
            if v is not None:
                if not isinstance(v, list):
                    setattr(p, t.localname, [v])
                getattr(p, t.localname).append(cd)
            else:
                setattr(p, t.localname, cd)
    return p